@@ -1,10 +1,28 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-
 import sys
+
 import click
-from .parseutils import is_destructive
+
+from mycli.packages.parseutils import is_destructive
+
+
+class ConfirmBoolParamType(click.ParamType):
+    name = "confirmation"
+
+    def convert(self, value, param, ctx):
+        if isinstance(value, bool):
+            return bool(value)
+        value = value.lower()
+        if value in ("yes", "y"):
+            return True
+        elif value in ("no", "n"):
+            return False
+        self.fail("%s is not a valid boolean" % value, param, ctx)
+
+    def __repr__(self):
+        return "BOOL"
+
+
+BOOLEAN_TYPE = ConfirmBoolParamType()
 
 
 def confirm_destructive_query(queries):
@@ -16,10 +34,9 @@ def confirm_destructive_query(queries):
     * False if the query is destructive and the user doesn't want to proceed.
 
     """
-    prompt_text = ("You're about to run a destructive command.\n"
-                   "Do you want to proceed? (y/n)")
+    prompt_text = "You're about to run a destructive command.\nDo you want to proceed? (y/n)"
     if is_destructive(queries) and sys.stdin.isatty():
-        return prompt(prompt_text, type=bool)
+        return prompt(prompt_text, type=BOOLEAN_TYPE)
 
 
 def confirm(*args, **kwargs):
