@@ -1,10 +1,14 @@
 __all__ = []
 
+
 def export(defn):
     """Decorator to explicitly mark functions that are exposed in a lib."""
     globals()[defn.__name__] = defn
     __all__.append(defn.__name__)
     return defn
 
-from . import dbcommands
-from . import iocommands
+
+from mycli.packages.special import (
+    dbcommands,  # noqa: E402 F401
+    iocommands,  # noqa: E402 F401
+)
